<?php

namespace PiiuHelmes\Model;

use PiiuHelmes\Exception\InvalidSectorDataException;

class Sector
{
    /**
     * @var int
     */
    protected $id;

    /**
     * @var string
     */
    protected $name;

    /**
     * @var int
     */
    protected $parentId;

    /**
     * @var int
     */
    protected $indent = 0;

    /**
     * @param array $rawData
     * @throws InvalidSectorDataException
     */
    public function __construct(array $rawData)
    {
        if (empty($rawData['id']) || empty($rawData['name'])) {
            throw new InvalidSectorDataException('All sectors must have ID and name');
        }
        $this->id = $rawData['id'];
        $this->name = $rawData['name'];
        $this->parentId = $rawData['parent'] ?? null;
        $this->indent = $rawData['indent'] ?? 0;
    }

    /**
     * @return int
     */
    public function getId(): int
    {
        return $this->id;
    }

    /**
     * @param int $id
     */
    public function setId(int $id)
    {
        $this->id = $id;
    }

    /**
     * @return string
     */
    public function getName(): string
    {
        return $this->name;
    }

    /**
     * @param string $name
     */
    public function setName(string $name)
    {
        $this->name = $name;
    }

    /**
     * @return int|null
     */
    public function getParentId(): ?int
    {
        return $this->parentId;
    }

    /**
     * @param int|null $parentId
     */
    public function setParentId(?int $parentId)
    {
        $this->parentId = $parentId;
    }

    /**
     * @return int
     */
    public function getIndent(): int
    {
        return $this->indent;
    }

    /**
     * @param int $indent
     */
    public function setIndent(int $indent)
    {
        $this->indent = $indent;
    }


}