<?php

namespace PiiuHelmes\Repository;

use Doctrine\DBAL\Connection;
use PiiuHelmes\Exception\NoSectorsException;
use PiiuHelmes\Model\Sector;

class SectorRepository
{
    /**
     * @var Connection
     */
    protected $db;

    /**
     * @param Connection $db
     */
    public function __construct(Connection $db)
    {
        $this->db = $db;
    }

    /**
     * @return array|Sector[]
     * @throws NoSectorsException
     */
    public function getList()
    {
        $data = $this->getByParent(null);
        if (empty($data)) {
            throw new NoSectorsException('No sectors found');
        }
        return $data;
    }

    /**
     * @param array $data
     * @param int|null $parent
     * @param int $indent
     * @return Sector[]
     */
    private function getByParent(?int $parent = null, int $indent = 0, array $data = []) : array
    {
        if ($parent) {
            $sql = "SELECT * FROM sectors where parent = ? ORDER BY name;";
            $sectors = $this->db->fetchAll($sql, [$parent]);
        } else {
            $sectors = $this->db->fetchAll("SELECT * FROM sectors where parent IS NULL ORDER BY name;");
        }

        foreach ($sectors as $rawSectorData) {
            $sector = new Sector($rawSectorData);
            $sector->setIndent($indent);
            $data[] = $sector;
            $data = $this->getByParent($sector->getId(), $indent+1, $data);
        }
        return $data;
    }
}