<?php

namespace PiiuHelmes\Repository;

use Doctrine\DBAL\Connection;
use PiiuHelmes\Model\User;
use PiiuHelmes\Exception\NoMatchingUserException;

class UserRepository
{
    /**
     * @var Connection
     */
    protected $db;

    /**
     * @param Connection $db
     */
    public function __construct(Connection $db)
    {
        $this->db = $db;
    }

    /**
     * @param int $id
     * @return User
     * @throws NoMatchingUserException
     */
    public function getUserById(int $id)
    {
        $sql = "SELECT users.id, users.name, users.agree, user_sectors.sector_id FROM users 
                LEFT JOIN user_sectors ON users.id = user_sectors.user_id WHERE users.id = ?;";
        $data = $this->db->fetchAll($sql, [$id]);
        if (empty($data)) {
            throw new NoMatchingUserException('No user found');
        }
        $user = new User($data[0]);
        $user->setSectors(array_map(function ($row) {
            return $row['sector_id'];
        }, $data));
        return $user;
    }

    /**
     * @param User $user
     * @return User
     */
    public function saveUser(User $user) : User
    {
        if ($user->hasId()) {
            $this->updateUser($user);
        } else {
            $user->setId($this->addUser($user));
        }
        $this->setSectors($user);

        return $user;
    }

    /**
     * @param User $user
     * @return int
     */
    private function addUser(User $user) : int
    {
        $statement = $this->db->prepare("INSERT INTO users (name,agree) VALUES (:name,:agree)");
        $statement->bindValue('name', $user->getName());
        $statement->bindValue('agree', $user->isAgree());
        $statement->execute();
        return $this->db->lastInsertId();
    }

    /**
     * @param User $user
     */
    private function updateUser(User $user)
    {
        $statement = $this->db->prepare("UPDATE users SET name=:name, agree=:agree WHERE id = :id");
        $statement->bindValue('name', $user->getName());
        $statement->bindValue('agree', $user->isAgree());
        $statement->bindValue('id', $user->getId());
        $statement->execute();
    }

    /**
     * @param User $user
     */
    private function setSectors(User $user)
    {
        $this->deleteAllSectors($user);
        foreach ($user->getSectors() as $sectorId) {
            $statement = $this->db->prepare("INSERT INTO user_sectors (user_id, sector_id) VALUES (:userId,:sectorId)");
            $statement->bindValue('userId', $user->getId());
            $statement->bindValue('sectorId', $sectorId);
            $statement->execute();
        }
    }

    /**
     * @param User $user
     */
    private function deleteAllSectors(User $user)
    {
        $statement = $this->db->prepare("DELETE FROM user_sectors WHERE user_id = :userId");
        $statement->bindValue('userId', $user->getId());
        $statement->execute();
    }
}